#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ------------
#  Change Log 
# ------------
# Rel 1.0: /22-12-2023/ Initial release 
#          This is a live preview overlay of the path-waves-0.2 plugin by Ofnuts (must be installed). https://sourceforge.net/projects/gimp-path-tools/files/scripts/
#          Renders a wave along the BOTTOM path.
#          Live Preview (gtk2) according to the idea of Tin Tran http://gimpchat.com/viewtopic.php?f=9&t=9586#p284974
# Rel 1.1  /24-12-2023/ added option: "Visibility (final)": ["No Change", "Only TOP", "Only BOTTOM", "TOP & BOTTOM", "All Visible", "All Invisible"]
# Rel 1.2  /03-01-2024/ added code by David Madsen: changed update policy of sliders to DISCONTINUOUS (this prevents the plugin from updating while moving the sliders).

from gimpfu import *
import gtk, gimpui
gimpui.gimp_ui_init ()

# Global variables to store the parameters used for our effect/work to show preview or actual layer when user OK it
global_param = 20
global_param1 = 20 #in this example it's radius
global_param2 = 0   #in this example it's samples

proceed_with_changes = False
proceed_with_changes_p = False
proceed_with_changes_d = False

image = 0 #we'll set these when dialog() is called so that we can access them later
drawable = 0
has_preview = False
preview_layer = 0

def apply_effect(layer): #function to do work on either preview layer or actual drawable when user clicks OK
    global image

    wave_length	= global_param
    amplitude_wave= global_param1

    asymmetry= global_param2
    if proceed_with_changes_p==0:	
        split=0
    if proceed_with_changes_p==1:	
        split=1
		
    if selected_option_2== "None" :
        Wavelength=0	
    if selected_option_2== "1/4" :
        Wavelength=1
    if selected_option_2== "1/2" :
        Wavelength=2
    if selected_option_2== "3/4" :
        Wavelength=3


		
    active_vectors=image.vectors[-1]

    if selected_option== "Sine":
        preview_layer=pdb.python_fu_path_periodic_sine_wave(image,active_vectors,wave_length,amplitude_wave,Wavelength, asymmetry, split)
	
    if selected_option== "Ellipse":
        pdb.python_fu_path_periodic_ellipse_wave(image,active_vectors,wave_length,amplitude_wave,Wavelength, asymmetry, split)
		
    if selected_option== "Circle":
        pdb.python_fu_path_periodic_circle_wave(image,active_vectors,wave_length,amplitude_wave,Wavelength, asymmetry, split)
		
    if selected_option== "Square":
        pdb.python_fu_path_periodic_square_wave(image,active_vectors,wave_length,amplitude_wave,Wavelength, asymmetry, split)
		
    if selected_option== "Triangle":
        pdb.python_fu_path_periodic_triangle_wave(image,active_vectors,wave_length,amplitude_wave,Wavelength, asymmetry, split)

# Visibility "No Change", "Only TOP", "Only BOTTOM","TOP & BOTTOM", "All Visible", "All Invisible"
    if selected_option_3=="Only TOP":
        for vector in image.vectors:
            pdb.gimp_item_set_visible(vector, 0)
        pdb.gimp_item_set_visible(image.vectors[0], 1)	
    if selected_option_3=="Only BOTTOM":
        for vector in image.vectors:
            pdb.gimp_item_set_visible(vector, 0)
        pdb.gimp_item_set_visible(image.vectors[-1], 1)
    if selected_option_3=="TOP & BOTTOM":
        for vector in image.vectors:
            pdb.gimp_item_set_visible(vector, 0)
        pdb.gimp_item_set_visible(image.vectors[0], 1)				
        pdb.gimp_item_set_visible(image.vectors[-1], 1)	
    if selected_option_3=="All Visible":
        for vector in image.vectors:
            pdb.gimp_item_set_visible(vector, 1)
    if selected_option_3=="All Invisible":
        for vector in image.vectors:
            pdb.gimp_item_set_visible(vector, 0)			
			
def apply_final(layer): #wrapper to apply effect on final and remove preview_layer meant to be called by on_ok_button_clicked
    global preview_layer
    if has_preview:
        pdb.gimp_image_remove_vectors(image,image.vectors[0])


# Function to update the live preview
def update_live_preview(): #this is called everytime some parameter changes
    global global_param, global_param1, global_param2, global_param3, global_param4, global_param5, global_param6, global_param17
    global image,drawable, vector
    global has_preview,preview_layer #deal with preview layer
   
    # Apply your plugin's effect using the current parameters
    # Use global_param1 and global_param2 to access the user's inputs
      


    #debug message
    #pdb.gimp_message(str(global_param1)+","+str(global_param2)+","+str(global_param3)+","+str(global_param4))

    if proceed_with_changes==1:	
        apply_effect(preview_layer)
        pdb.gimp_image_remove_vectors(image,image.vectors[0])
    if proceed_with_changes==0 and proceed_with_changes_d==1:
        pdb.gimp_image_remove_vectors(image,image.vectors[0])
    if proceed_with_changes==0 and proceed_with_changes_d==0:
        pass	
    if proceed_with_changes==1:	
        apply_effect(preview_layer)        
   
		
            #image.vectors[0].visible = True		

def dialog(image_, drawable_):
    global image, drawable
    #save these for updates
    image = image_ #for speed and also when user undo it doesn't see our preview creations/deletions
    drawable = drawable_ 

    dialog = gtk.Dialog("MrQ Path Waves by Ofn Live Preview", None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
    dialog.set_default_size(400, 100)
	
    #-------------------------------------------------------------
	
    hbox0 = gtk.HBox()
    dialog.vbox.pack_start(hbox0, expand=True, fill=True)
    pattern_label = gtk.Label("\nThis is a Live Preview overlay of the ofn path-waves-0.2.py (plugin must be installed).\n                               Renders a wave along the BOTTOM path.\n               It is recommended to change the parameters when Preview = No.\n")	
    hbox0.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------
	
    # Combo box
    hbox7 = gtk.HBox()
    dialog.vbox.pack_start(hbox7, expand=True, fill=True)
    label7 = gtk.Label("Wave Type:               ")
    hbox7.pack_start(label7, expand=False, fill=False, padding=5)   

    combo_box = gtk.ComboBox()
    hbox7.pack_start(combo_box, expand=False, fill=False, padding=5)

    # Create a ListStore model for ComboBox
    list_store = gtk.ListStore(str)
    options = ["Sine", "Ellipse", "Circle", "Square", "Triangle"]  # Add your options here
    for option in options:
        list_store.append([option])

    # Set the model for ComboBox
    combo_box.set_model(list_store)

    # Create a CellRendererText to render the text in the ComboBox
    cell_renderer = gtk.CellRendererText()

    # Pack the CellRendererText into the ComboBox
    combo_box.pack_start(cell_renderer, True)
    combo_box.add_attribute(cell_renderer, 'text', 0)

    # Set up the "changed" signal handler   
    combo_box.connect("changed", on_combobox_changed)

    # Set the default option
    default_option = "Sine"
    default_iter = list_store.get_iter_first()
    while default_iter is not None:
        if list_store.get_value(default_iter, 0) == default_option:
            combo_box.set_active_iter(default_iter)
            break
        default_iter = list_store.iter_next(default_iter)

    # -------------------------------------------------------------
	
    hbox = gtk.HBox()
    dialog.vbox.pack_start(hbox, expand=True, fill=True)
    label = gtk.Label("\nWavelength (pixels):")
    hbox.pack_start(label, expand=False, fill=False, padding=5)
    adjustment = gtk.Adjustment(value=20, lower=1, upper=1000, step_incr=1, page_incr=0)
    param_scale = gtk.HScale(adjustment)
    param_scale.set_digits(0)  # Display only integers
    hbox.pack_start(param_scale, expand=True, fill=True, padding=5)
    label = gtk.SpinButton(adjustment,0, 0 )
    hbox.pack_start(label, expand=False, fill=False, padding=5)	
    param_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######		
    param_scale.connect("value-changed", on_param_changed)

    # -------------------------------------------------------------
	
    hbox1 = gtk.HBox()
    dialog.vbox.pack_start(hbox1, expand=True, fill=True)
    label1 = gtk.Label("\nAmplitude (pixels):   ")
    hbox1.pack_start(label1, expand=False, fill=False, padding=5)
    adjustment1 = gtk.Adjustment(value=20, lower=1, upper=1000, step_incr=1, page_incr=0)
    param1_scale = gtk.HScale(adjustment=adjustment1)
    param1_scale.set_digits(0)  # Display only integers
    hbox1.pack_start(param1_scale, expand=True, fill=True, padding=5)
    label1 = gtk.SpinButton(adjustment1,0, 0 )
    hbox1.pack_start(label1, expand=False, fill=False, padding=5)
    param1_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######		
    param1_scale.connect("value-changed", on_param1_changed)

    #-------------------------------------------------------------

    hbox40 = gtk.HBox()
    dialog.vbox.pack_start(hbox40, expand=True, fill=True)
    pattern_label = gtk.Label("")	
    hbox40.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------
    # Combo box
    hbox7 = gtk.HBox()
    dialog.vbox.pack_start(hbox7, expand=True, fill=True)
    label7 = gtk.Label("Wavelength shift:      ")
    hbox7.pack_start(label7, expand=False, fill=False, padding=5)   

    # Create a ComboBox
    combo_box = gtk.ComboBox()
    hbox7.pack_start(combo_box, expand=False, fill=False, padding=5)

    # Create a ListStore model for ComboBox
    list_store = gtk.ListStore(str)
    options = ["None","1/4","1/2","3/4"] # Add your options here
    for option in options:
        list_store.append([option])

    # Set the model for ComboBox
    combo_box.set_model(list_store)

    # Create a CellRendererText to render the text in the ComboBox
    cell_renderer = gtk.CellRendererText()

    # Pack the CellRendererText into the ComboBox
    combo_box.pack_start(cell_renderer, True)
    combo_box.add_attribute(cell_renderer, 'text', 0)

    # Set up the "changed" signal handler   
    combo_box.connect("changed", on_combobox_changed_2)

    # Set the default option
    default_option = "None"
    default_iter = list_store.get_iter_first()
    while default_iter is not None:
        if list_store.get_value(default_iter, 0) == default_option:
            combo_box.set_active_iter(default_iter)
            break
        default_iter = list_store.iter_next(default_iter)
    #-------------------------------------------------------------

    hbox41 = gtk.HBox()
    dialog.vbox.pack_start(hbox41, expand=True, fill=True)
    pattern_label = gtk.Label("")	
    hbox41.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------
	
    hbox6 = gtk.HBox()
    dialog.vbox.pack_start(hbox6, expand=True, fill=True)
    label6 = gtk.Label("Split top and bottom:")
    hbox6.pack_start(label6, expand=False, fill=False, padding=5)   
    toggle_button = gtk.ToggleButton()
    update_toggle_button_label(toggle_button)
    toggle_button.connect("toggled", on_toggle_button_toggled_p)
    hbox6.pack_start(toggle_button, expand=False, fill=False, padding=5) 
	
    #-------------------------------------------------------------

    hbox12 = gtk.HBox()
    dialog.vbox.pack_start(hbox12, expand=True, fill=True)
    toggle_button = gtk.ToggleButton()
    dialog.action_area.add(toggle_button)	
    update_toggle_button_label(toggle_button)
    toggle_button.connect("toggled", on_toggle_button_toggled)
    hbox12.pack_start(toggle_button, expand=False, fill=False, padding=5) 
	
    #-------------------------------------------------------------
	
    hbox0 = gtk.HBox()
    dialog.vbox.pack_start(hbox0, expand=True, fill=True)
    pattern_label = gtk.Label("                              ")
    dialog.action_area.add(pattern_label)	
    hbox0.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------
	
    hbox2 = gtk.HBox()
    dialog.vbox.pack_start(hbox2, expand=True, fill=True)
    label2 = gtk.Label("\nAsymmetry:              ")
    hbox2.pack_start(label2, expand=False, fill=False, padding=5)
    adjustment2 = gtk.Adjustment(value=0, lower=-49, upper=49, step_incr=1, page_incr=0)
    param2_scale = gtk.HScale(adjustment=adjustment2)
    param2_scale.set_digits(0)  # Display only integers
    hbox2.pack_start(param2_scale, expand=True, fill=True, padding=5)
    label2 = gtk.SpinButton(adjustment2,0, 0 )
    hbox2.pack_start(label2, expand=False, fill=False, padding=5)
    param2_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######		
    param2_scale.connect("value-changed", on_param2_changed)

    #-------------------------------------------------------------

    hbox42 = gtk.HBox()
    dialog.vbox.pack_start(hbox42, expand=True, fill=True)
    pattern_label = gtk.Label("")	
    hbox42.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------	
    # Combo box
    hbox17 = gtk.HBox()
    dialog.vbox.pack_start(hbox17, expand=True, fill=True)
    label17 = gtk.Label("Visibility (final):         ")
    hbox17.pack_start(label17, expand=False, fill=False, padding=5)   

    combo_box = gtk.ComboBox()
    hbox17.pack_start(combo_box, expand=False, fill=False, padding=5)

    # Create a ListStore model for ComboBox
    list_store = gtk.ListStore(str)
    options = ["No Change", "Only TOP", "Only BOTTOM","TOP & BOTTOM", "All Visible", "All Invisible"]  # Add your options here
    for option in options:
        list_store.append([option])

    # Set the model for ComboBox
    combo_box.set_model(list_store)

    # Create a CellRendererText to render the text in the ComboBox
    cell_renderer = gtk.CellRendererText()

    # Pack the CellRendererText into the ComboBox
    combo_box.pack_start(cell_renderer, True)
    combo_box.add_attribute(cell_renderer, 'text', 0)

    # Set up the "changed" signal handler   
    combo_box.connect("changed", on_combobox_changed_3)

    # Set the default option
    default_option = "No Change"
    default_iter = list_store.get_iter_first()
    while default_iter is not None:
        if list_store.get_value(default_iter, 0) == default_option:
            combo_box.set_active_iter(default_iter)
            break
        default_iter = list_store.iter_next(default_iter)

    # -------------------------------------------------------------
    # -------------------------------------------------------------
    hbox20 = gtk.HBox()
    dialog.vbox.pack_start(hbox20, expand=False, fill=False)
    pattern_label = gtk.Label("\n                                                                            Delete TOP path                          \n          Preview:                                                    (if Preview=No):")
    #dialog.action_area.add(pattern_label)	
    hbox20.pack_start(pattern_label, expand=False, fill=False, padding=1)
	
    # -------------------------------------------------------------
    hbox121 = gtk.HBox()
    dialog.vbox.pack_start(hbox121, expand=True, fill=True)
    toggle_button_d = gtk.ToggleButton()
    dialog.action_area.add(toggle_button_d)	
    update_toggle_button_label(toggle_button_d)
    toggle_button_d.connect("toggled", on_toggle_button_toggled_d)
    hbox121.pack_start(toggle_button_d, expand=False, fill=False, padding=5) 
	
    # -------------------------------------------------------------

    # Add an OK button
    ok_button = dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
    ok_button.connect("clicked", on_ok_button_clicked)
    # -------------------------------------------------------------	
	
    # Show the dialog
    dialog.show_all()
    dialog.run()
	
#######################################################
	
def on_combobox_changed(combobox):
    global selected_option
    model = combobox.get_model()
    active_iter = combobox.get_active_iter()

    if active_iter:
        selected_option = model.get_value(active_iter, 0)
    update_live_preview()
	
def on_combobox_changed_2(combobox):
    global selected_option_2
    model = combobox.get_model()
    active_iter = combobox.get_active_iter()

    if active_iter:
        selected_option_2 = model.get_value(active_iter, 0)
    update_live_preview()

def on_combobox_changed_3(combobox):
    global selected_option_3
    model = combobox.get_model()
    active_iter = combobox.get_active_iter()

    if active_iter:
        selected_option_3 = model.get_value(active_iter, 0)
    update_live_preview()
	
def on_param_changed(scale):
    global global_param
    global_param = scale.get_value()
    update_live_preview()

def on_param1_changed(scale):
    global global_param1
    global_param1 = scale.get_value()
    update_live_preview()

def on_param2_changed(scale):
    global global_param2
    global_param2 = scale.get_value()
    update_live_preview()
	
def update_toggle_button_label_p(button):
    label_s = "Yes" if proceed_with_changes_p else "No"
    button.set_label(label_s)

def on_toggle_button_toggled_p(button):
    global proceed_with_changes_p
    proceed_with_changes_p = not proceed_with_changes_p
    update_toggle_button_label_p(button)	
    update_live_preview()
	
def on_param3_changed(scale):
    global global_param3
    global_param3 = scale.get_value()
    update_live_preview()

def on_param4_changed(scale):
    global global_param4
    global_param4 = scale.get_value()
    update_live_preview()

def on_toggle_button_toggled(button):
    global proceed_with_changes
    proceed_with_changes = not proceed_with_changes
    update_toggle_button_label(button)

def update_toggle_button_label(button):
    label = "Click to Stop" if proceed_with_changes else "           No             "
    button.set_label(label)	
    update_live_preview()	

def on_toggle_button_toggled_d(button):
    global proceed_with_changes_d
    proceed_with_changes_d = not proceed_with_changes_d
    update_toggle_button_label_d(button)	
    update_live_preview()
	
def update_toggle_button_label_d(button):
    label_d = "Yes" if proceed_with_changes_d else "No"
    button.set_label(label_d)
	
def on_ok_button_clicked(button, data=None):
    global drawable
    #apply_final(drawable)
    button.get_toplevel().destroy() #destroys the gtk dialog window

register(
    "python_fu_ofn_path_waves_live_preview",
    "To render a wave along a path Live Preview",
    "Create a live preview of an effect",
    "MrQ",
    "GPL",
    "2023",
    "<Image>/Filters/Path/Path Waves LP...",  # Menu location
    "*",  # Image type
    [],
    [],
    dialog
)

main()